/* zlib_name_mangling.h has been automatically generated from
 * zlib_name_mangling.h.in because ZLIB_SYMBOL_PREFIX was set.
 */

#ifndef ZLIB_NAME_MANGLING_H
#define ZLIB_NAME_MANGLING_H

/* all linked symbols and init macros */
#define _dist_code            LIBRAPIDARCHIVE__dist_code
#define _length_code          LIBRAPIDARCHIVE__length_code
#define _tr_align             LIBRAPIDARCHIVE__tr_align
#define _tr_flush_bits        LIBRAPIDARCHIVE__tr_flush_bits
#define _tr_flush_block       LIBRAPIDARCHIVE__tr_flush_block
#define _tr_init              LIBRAPIDARCHIVE__tr_init
#define _tr_stored_block      LIBRAPIDARCHIVE__tr_stored_block
#define _tr_tally             LIBRAPIDARCHIVE__tr_tally
#define adler32               LIBRAPIDARCHIVE_adler32
#define adler32_combine       LIBRAPIDARCHIVE_adler32_combine
#define adler32_combine64     LIBRAPIDARCHIVE_adler32_combine64
#define adler32_z             LIBRAPIDARCHIVE_adler32_z
#ifndef Z_SOLO
#  define compress              LIBRAPIDARCHIVE_compress
#  define compress2             LIBRAPIDARCHIVE_compress2
#  define compressBound         LIBRAPIDARCHIVE_compressBound
#endif
#define crc32                 LIBRAPIDARCHIVE_crc32
#define crc32_combine         LIBRAPIDARCHIVE_crc32_combine
#define crc32_combine64       LIBRAPIDARCHIVE_crc32_combine64
#define crc32_combine_gen     LIBRAPIDARCHIVE_crc32_combine_gen
#define crc32_combine_gen64   LIBRAPIDARCHIVE_crc32_combine_gen64
#define crc32_combine_op      LIBRAPIDARCHIVE_crc32_combine_op
#define crc32_z               LIBRAPIDARCHIVE_crc32_z
#define deflate               LIBRAPIDARCHIVE_deflate
#define deflateBound          LIBRAPIDARCHIVE_deflateBound
#define deflateCopy           LIBRAPIDARCHIVE_deflateCopy
#define deflateEnd            LIBRAPIDARCHIVE_deflateEnd
#define deflateGetDictionary  LIBRAPIDARCHIVE_deflateGetDictionary
#define deflateInit           LIBRAPIDARCHIVE_deflateInit
#define deflateInit2          LIBRAPIDARCHIVE_deflateInit2
#define deflateInit2_         LIBRAPIDARCHIVE_deflateInit2_
#define deflateInit_          LIBRAPIDARCHIVE_deflateInit_
#define deflateParams         LIBRAPIDARCHIVE_deflateParams
#define deflatePending        LIBRAPIDARCHIVE_deflatePending
#define deflatePrime          LIBRAPIDARCHIVE_deflatePrime
#define deflateReset          LIBRAPIDARCHIVE_deflateReset
#define deflateResetKeep      LIBRAPIDARCHIVE_deflateResetKeep
#define deflateSetDictionary  LIBRAPIDARCHIVE_deflateSetDictionary
#define deflateSetHeader      LIBRAPIDARCHIVE_deflateSetHeader
#define deflateTune           LIBRAPIDARCHIVE_deflateTune
#define deflate_copyright     LIBRAPIDARCHIVE_deflate_copyright
#define fill_window           LIBRAPIDARCHIVE_fill_window
#define fixedtables           LIBRAPIDARCHIVE_fixedtables
#define flush_pending         LIBRAPIDARCHIVE_flush_pending
#define get_crc_table         LIBRAPIDARCHIVE_get_crc_table
#ifndef Z_SOLO
#  define gz_error              LIBRAPIDARCHIVE_gz_error
#  define gz_strwinerror        LIBRAPIDARCHIVE_gz_strwinerror
#  define gzbuffer              LIBRAPIDARCHIVE_gzbuffer
#  define gzclearerr            LIBRAPIDARCHIVE_gzclearerr
#  define gzclose               LIBRAPIDARCHIVE_gzclose
#  define gzclose_r             LIBRAPIDARCHIVE_gzclose_r
#  define gzclose_w             LIBRAPIDARCHIVE_gzclose_w
#  define gzdirect              LIBRAPIDARCHIVE_gzdirect
#  define gzdopen               LIBRAPIDARCHIVE_gzdopen
#  define gzeof                 LIBRAPIDARCHIVE_gzeof
#  define gzerror               LIBRAPIDARCHIVE_gzerror
#  define gzflush               LIBRAPIDARCHIVE_gzflush
#  define gzfread               LIBRAPIDARCHIVE_gzfread
#  define gzfwrite              LIBRAPIDARCHIVE_gzfwrite
#  define gzgetc                LIBRAPIDARCHIVE_gzgetc
#  define gzgetc_               LIBRAPIDARCHIVE_gzgetc_
#  define gzgets                LIBRAPIDARCHIVE_gzgets
#  define gzoffset              LIBRAPIDARCHIVE_gzoffset
#  define gzoffset64            LIBRAPIDARCHIVE_gzoffset64
#  define gzopen                LIBRAPIDARCHIVE_gzopen
#  define gzopen64              LIBRAPIDARCHIVE_gzopen64
#  ifdef _WIN32
#    define gzopen_w              LIBRAPIDARCHIVE_gzopen_w
#  endif
#  define gzprintf              LIBRAPIDARCHIVE_gzprintf
#  define gzputc                LIBRAPIDARCHIVE_gzputc
#  define gzputs                LIBRAPIDARCHIVE_gzputs
#  define gzread                LIBRAPIDARCHIVE_gzread
#  define gzrewind              LIBRAPIDARCHIVE_gzrewind
#  define gzseek                LIBRAPIDARCHIVE_gzseek
#  define gzseek64              LIBRAPIDARCHIVE_gzseek64
#  define gzsetparams           LIBRAPIDARCHIVE_gzsetparams
#  define gztell                LIBRAPIDARCHIVE_gztell
#  define gztell64              LIBRAPIDARCHIVE_gztell64
#  define gzungetc              LIBRAPIDARCHIVE_gzungetc
#  define gzvprintf             LIBRAPIDARCHIVE_gzvprintf
#  define gzwrite               LIBRAPIDARCHIVE_gzwrite
#endif
#define inflate               LIBRAPIDARCHIVE_inflate
#define inflateBack           LIBRAPIDARCHIVE_inflateBack
#define inflateBackEnd        LIBRAPIDARCHIVE_inflateBackEnd
#define inflateBackInit       LIBRAPIDARCHIVE_inflateBackInit
#define inflateBackInit_      LIBRAPIDARCHIVE_inflateBackInit_
#define inflateCodesUsed      LIBRAPIDARCHIVE_inflateCodesUsed
#define inflateCopy           LIBRAPIDARCHIVE_inflateCopy
#define inflateEnd            LIBRAPIDARCHIVE_inflateEnd
#define inflateGetDictionary  LIBRAPIDARCHIVE_inflateGetDictionary
#define inflateGetHeader      LIBRAPIDARCHIVE_inflateGetHeader
#define inflateInit           LIBRAPIDARCHIVE_inflateInit
#define inflateInit2          LIBRAPIDARCHIVE_inflateInit2
#define inflateInit2_         LIBRAPIDARCHIVE_inflateInit2_
#define inflateInit_          LIBRAPIDARCHIVE_inflateInit_
#define inflateMark           LIBRAPIDARCHIVE_inflateMark
#define inflatePrime          LIBRAPIDARCHIVE_inflatePrime
#define inflateReset          LIBRAPIDARCHIVE_inflateReset
#define inflateReset2         LIBRAPIDARCHIVE_inflateReset2
#define inflateResetKeep      LIBRAPIDARCHIVE_inflateResetKeep
#define inflateSetDictionary  LIBRAPIDARCHIVE_inflateSetDictionary
#define inflateSync           LIBRAPIDARCHIVE_inflateSync
#define inflateSyncPoint      LIBRAPIDARCHIVE_inflateSyncPoint
#define inflateUndermine      LIBRAPIDARCHIVE_inflateUndermine
#define inflateValidate       LIBRAPIDARCHIVE_inflateValidate
#define inflate_copyright     LIBRAPIDARCHIVE_inflate_copyright
#define inflate_ensure_window LIBRAPIDARCHIVE_inflate_ensure_window
#define inflate_fast          LIBRAPIDARCHIVE_inflate_fast
#define inflate_table         LIBRAPIDARCHIVE_inflate_table
#define read_buf              LIBRAPIDARCHIVE_read_buf
#ifndef Z_SOLO
#  define uncompress            LIBRAPIDARCHIVE_uncompress
#  define uncompress2           LIBRAPIDARCHIVE_uncompress2
#endif
#define zError                LIBRAPIDARCHIVE_zError
#ifndef Z_SOLO
#  define zcalloc               LIBRAPIDARCHIVE_zcalloc
#  define zcfree                LIBRAPIDARCHIVE_zcfree
#endif
#define zlibCompileFlags      LIBRAPIDARCHIVE_zlibCompileFlags
#define zlibVersion           LIBRAPIDARCHIVE_zlibVersion

/* all zlib typedefs in zlib.h and zconf.h */
#define Byte                  LIBRAPIDARCHIVE_Byte
#define Bytef                 LIBRAPIDARCHIVE_Bytef
#define alloc_func            LIBRAPIDARCHIVE_alloc_func
#define charf                 LIBRAPIDARCHIVE_charf
#define free_func             LIBRAPIDARCHIVE_free_func
#ifndef Z_SOLO
#  define gzFile                LIBRAPIDARCHIVE_gzFile
#endif
#define gz_header             LIBRAPIDARCHIVE_gz_header
#define gz_headerp            LIBRAPIDARCHIVE_gz_headerp
#define in_func               LIBRAPIDARCHIVE_in_func
#define intf                  LIBRAPIDARCHIVE_intf
#define out_func              LIBRAPIDARCHIVE_out_func
#define uInt                  LIBRAPIDARCHIVE_uInt
#define uIntf                 LIBRAPIDARCHIVE_uIntf
#define uLong                 LIBRAPIDARCHIVE_uLong
#define uLongf                LIBRAPIDARCHIVE_uLongf
#define voidp                 LIBRAPIDARCHIVE_voidp
#define voidpc                LIBRAPIDARCHIVE_voidpc
#define voidpf                LIBRAPIDARCHIVE_voidpf

/* all zlib structs in zlib.h and zconf.h */
#define gz_header_s           LIBRAPIDARCHIVE_gz_header_s
#define internal_state        LIBRAPIDARCHIVE_internal_state

/* all zlib structs in zutil.h */
#define z_errmsg              LIBRAPIDARCHIVE_z_errmsg
#define z_vstring             LIBRAPIDARCHIVE_z_vstring
#define zlibng_version        LIBRAPIDARCHIVE_zlibng_version

/* zlib-ng specific symbols */
#define zng_alloc_aligned     LIBRAPIDARCHIVE_zng_alloc_aligned
#define zng_free_aligned      LIBRAPIDARCHIVE_zng_free_aligned

#endif /* ZLIB_NAME_MANGLING_H */
